@@ -3,7 +3,7 @@ package Yote::Spiderpup;
 use strict;
 use warnings;
 
-our $VERSION = '0.01';
+our $VERSION = '0.02';
 
 use CSS::LESSp;
 use Data::Dumper;
@@ -14,6 +14,8 @@ use File::Path qw(make_path);
 use File::Spec;
 use YAML;
 
+use Yote::Spiderpup::SFC qw(parse_sfc);
+
 use Yote::Spiderpup::Transform qw(
     transform_dollar_vars
     transform_expression
@@ -56,7 +58,7 @@ sub _scan_yaml_dir {
         my $path = File::Spec->catfile($dir, $entry);
         if (-d $path) {
             $self->_scan_yaml_dir($path, $changed_ref);
-        } elsif ($entry =~ /\.yaml$/) {
+        } elsif ($entry =~ /\.(yaml|pup)$/) {
             my $mtime = (stat($path))[9];
             if (!exists $self->{file_mtimes}{$path} || $self->{file_mtimes}{$path} != $mtime) {
                 $self->{file_mtimes}{$path} = $mtime;
@@ -84,6 +86,30 @@ sub is_dev_mode {
     return $ENV{SPIDERPUP_DEV} ? 1 : 0;
 }
 
+#----------------------------------------------------------------------
+# Component file helpers (.yaml / .pup)
+#----------------------------------------------------------------------
+
+sub _find_component_file {
+    my ($self, $dir, $name) = @_;
+    my $yaml = File::Spec->catfile($dir, "$name.yaml");
+    return $yaml if -f $yaml;
+    my $pup = File::Spec->catfile($dir, "$name.pup");
+    return $pup if -f $pup;
+    return undef;
+}
+
+sub _load_component_data {
+    my ($self, $file_path) = @_;
+    if ($file_path =~ /\.pup$/) {
+        open my $fh, '<', $file_path or die "Cannot open $file_path: $!";
+        my $content = do { local $/; <$fh> };
+        close $fh;
+        return parse_sfc($content);
+    }
+    return YAML::LoadFile($file_path);
+}
+
 #----------------------------------------------------------------------
 # Path resolution
 #----------------------------------------------------------------------
@@ -94,6 +120,7 @@ sub parse_page_path {
     $path =~ s|^/+||;
     $path =~ s|\.html$||;
     $path =~ s|\.yaml$||;
+    $path =~ s|\.pup$||;
 
     return 'index' if $path eq '' || $path eq 'index';
 
@@ -114,21 +141,22 @@ sub load_page {
     $path =~ s|^/+||;
     $path =~ s|\.html$||;
     $path =~ s|\.yaml$||;
+    $path =~ s|\.pup$||;
     $path = 'index' if $path eq '' || $path eq 'index.html';
 
-    my $yaml_file;
+    my $component_file;
 
     if ($path =~ s|^recipes/||) {
-        $yaml_file = File::Spec->catfile($self->{recipes_dir}, "$path.yaml");
+        $component_file = $self->_find_component_file($self->{recipes_dir}, $path);
     } else {
-        $yaml_file = File::Spec->catfile($self->{pages_dir}, "$path.yaml");
+        $component_file = $self->_find_component_file($self->{pages_dir}, $path);
     }
 
-    return undef unless -f $yaml_file;
+    return undef unless $component_file;
 
-    my $page_data = YAML::LoadFile($yaml_file);
-    my $relative_yaml_path = File::Spec->abs2rel($yaml_file, $self->{www_dir});
-    $page_data->{yaml_path} = $relative_yaml_path;
+    my $page_data = $self->_load_component_data($component_file);
+    my $relative_path = File::Spec->abs2rel($component_file, $self->{www_dir});
+    $page_data->{yaml_path} = $relative_path;
 
     return $page_data;
 }
@@ -138,6 +166,7 @@ sub resolve_recipe_name {
 
     $import_path =~ s|^/+||;
     $import_path =~ s|\.yaml$||;
+    $import_path =~ s|\.pup$||;
 
     my ($module_name, $is_page);
     if ($import_path =~ m|^recipes/(.+)$|) {
@@ -145,8 +174,8 @@ sub resolve_recipe_name {
         $is_page = 0;
     } else {
         $module_name = $import_path;
-        my $page_yaml = File::Spec->catfile($self->{pages_dir}, "$module_name.yaml");
-        $is_page = (-f $page_yaml) ? 1 : 0;
+        my $page_file = $self->_find_component_file($self->{pages_dir}, $module_name);
+        $is_page = $page_file ? 1 : 0;
     }
 
     my $class_name = $module_name;
@@ -183,14 +212,14 @@ sub collect_yaml_files {
     my ($self, $page_data, $page_name, $collected) = @_;
     $collected //= {};
 
-    my $yaml_file;
+    my $component_file;
     if ($page_name =~ /^recipes\/(.+)$/) {
-        $yaml_file = File::Spec->catfile($self->{recipes_dir}, "$1.yaml");
+        $component_file = $self->_find_component_file($self->{recipes_dir}, $1);
     } else {
-        $yaml_file = File::Spec->catfile($self->{pages_dir}, "$page_name.yaml");
+        $component_file = $self->_find_component_file($self->{pages_dir}, $page_name);
     }
-    if (-f $yaml_file) {
-        $collected->{$yaml_file} = (stat($yaml_file))[9];
+    if ($component_file) {
+        $collected->{$component_file} = (stat($component_file))[9];
     }
 
     my $imports = $page_data->{import} // {};
@@ -198,13 +227,15 @@ sub collect_yaml_files {
         my $import_path = $imports->{$namespace};
         $import_path =~ s|^/+||;
         $import_path =~ s|\.yaml$||;
+        $import_path =~ s|\.pup$||;
 
         my $import_file;
         if ($import_path =~ /^recipes\/(.+)$/) {
-            $import_file = File::Spec->catfile($self->{recipes_dir}, "$1.yaml");
+            $import_file = $self->_find_component_file($self->{recipes_dir}, $1);
         } else {
-            $import_file = File::Spec->catfile($self->{pages_dir}, "$import_path.yaml");
+            $import_file = $self->_find_component_file($self->{pages_dir}, $import_path);
         }
+        next unless $import_file;
         next if exists $collected->{$import_file};
 
         my $imported_page = $self->load_page($import_path);
@@ -469,11 +500,11 @@ CLASS
 sub compile_recipe {
     my ($self, $module_name) = @_;
 
-    my $yaml_file = File::Spec->catfile($self->{recipes_dir}, "$module_name.yaml");
-    return unless -f $yaml_file;
+    my $component_file = $self->_find_component_file($self->{recipes_dir}, $module_name);
+    return unless $component_file;
 
-    my $page_data = YAML::LoadFile($yaml_file);
-    $page_data->{yaml_path} = File::Spec->abs2rel($yaml_file, $self->{www_dir});
+    my $page_data = $self->_load_component_data($component_file);
+    $page_data->{yaml_path} = File::Spec->abs2rel($component_file, $self->{www_dir});
 
     my $class_name = $module_name;
     $class_name =~ s/-(.)/\U$1/g;
@@ -547,12 +578,12 @@ sub compile_recipe {
 
     my $meta_file = File::Spec->catfile($self->{js_dir}, "$module_name.js.meta");
     my %mtimes;
-    $mtimes{$yaml_file} = (stat($yaml_file))[9];
+    $mtimes{$component_file} = (stat($component_file))[9];
     for my $namespace (keys %$imports) {
         my $import_path = $imports->{$namespace};
         my ($dep_mod_name) = $self->resolve_recipe_name($import_path);
-        my $dep_yaml = File::Spec->catfile($self->{recipes_dir}, "$dep_mod_name.yaml");
-        $mtimes{$dep_yaml} = (stat($dep_yaml))[9] if -f $dep_yaml;
+        my $dep_file = $self->_find_component_file($self->{recipes_dir}, $dep_mod_name);
+        $mtimes{$dep_file} = (stat($dep_file))[9] if $dep_file;
     }
     open my $meta_fh, '>', $meta_file or die "Cannot write $meta_file: $!";
     print $meta_fh encode_json(\%mtimes);
@@ -594,11 +625,11 @@ sub compile_recipe_if_stale {
 sub compile_page_js {
     my ($self, $page_name) = @_;
 
-    my $yaml_file = File::Spec->catfile($self->{pages_dir}, "$page_name.yaml");
-    return unless -f $yaml_file;
+    my $component_file = $self->_find_component_file($self->{pages_dir}, $page_name);
+    return unless $component_file;
 
-    my $page_data = YAML::LoadFile($yaml_file);
-    $page_data->{yaml_path} = File::Spec->abs2rel($yaml_file, $self->{www_dir});
+    my $page_data = $self->_load_component_data($component_file);
+    $page_data->{yaml_path} = File::Spec->abs2rel($component_file, $self->{www_dir});
 
     my $class_name = $page_name;
     $class_name =~ s|/|_|g;
@@ -693,17 +724,13 @@ sub compile_page_js {
 
     my $meta_file = File::Spec->catfile($self->{js_dir}, 'pages', "$page_name.js.meta");
     my %mtimes;
-    $mtimes{$yaml_file} = (stat($yaml_file))[9];
+    $mtimes{$component_file} = (stat($component_file))[9];
     for my $namespace (keys %$imports) {
         my $import_path = $imports->{$namespace};
         my ($dep_name, undef, undef, $dep_is_page) = $self->resolve_recipe_name($import_path);
-        my $dep_yaml;
-        if ($dep_is_page) {
-            $dep_yaml = File::Spec->catfile($self->{pages_dir}, "$dep_name.yaml");
-        } else {
-            $dep_yaml = File::Spec->catfile($self->{recipes_dir}, "$dep_name.yaml");
-        }
-        $mtimes{$dep_yaml} = (stat($dep_yaml))[9] if -f $dep_yaml;
+        my $dep_dir = $dep_is_page ? $self->{pages_dir} : $self->{recipes_dir};
+        my $dep_file = $self->_find_component_file($dep_dir, $dep_name);
+        $mtimes{$dep_file} = (stat($dep_file))[9] if $dep_file;
     }
     open my $meta_fh, '>', $meta_file or die "Cannot write $meta_file: $!";
     print $meta_fh encode_json(\%mtimes);
@@ -1304,18 +1331,19 @@ sub run_server {
                     }
                 }
 
-                # Serve raw YAML files
-                if (!$response && $path =~ /\.yaml$/) {
-                    my $yaml_path = $path;
-                    $yaml_path =~ s~^/~~;
-                    my $yaml_file = File::Spec->catfile($self->{www_dir}, $yaml_path);
-                    if (-f $yaml_file) {
-                        open my $fh, '<', $yaml_file or die "Cannot open $yaml_file: $!";
+                # Serve raw YAML/PUP files
+                if (!$response && $path =~ /\.(yaml|pup)$/) {
+                    my $raw_path = $path;
+                    $raw_path =~ s~^/~~;
+                    my $raw_file = File::Spec->catfile($self->{www_dir}, $raw_path);
+                    if (-f $raw_file) {
+                        open my $fh, '<', $raw_file or die "Cannot open $raw_file: $!";
                         my $body = do { local $/; <$fh> };
                         close $fh;
+                        my $content_type = $raw_file =~ /\.pup$/ ? 'text/plain' : 'text/yaml';
                         my $content_length = length($body);
                         $response = "HTTP/1.1 200 OK\r\n";
-                        $response .= "Content-Type: text/yaml; charset=utf-8\r\n";
+                        $response .= "Content-Type: $content_type; charset=utf-8\r\n";
                         $response .= "Content-Length: $content_length\r\n";
                         $response .= "Connection: close\r\n";
                         $response .= "\r\n";
@@ -1360,9 +1388,9 @@ sub find_yaml_files {
         if (-d $path) {
             my $new_prefix = $prefix ? "$prefix/$entry" : $entry;
             $self->find_yaml_files($path, $new_prefix, $results);
-        } elsif ($entry =~ /\.yaml$/) {
+        } elsif ($entry =~ /\.(yaml|pup)$/) {
             my $name = $entry;
-            $name =~ s/\.yaml$//;
+            $name =~ s/\.(yaml|pup)$//;
             $name = $prefix ? "$prefix/$name" : $name;
             push @$results, $name;
         }
@@ -1381,12 +1409,12 @@ sub compile_all {
     # Phase 1: Compile all recipes to JS files
     print "--- Compiling recipes ---\n";
     opendir(my $mod_dh, $self->{recipes_dir}) or die "Cannot open $self->{recipes_dir}: $!";
-    my @module_files = sort grep { /\.yaml$/ } readdir($mod_dh);
+    my @module_files = sort grep { /\.(yaml|pup)$/ } readdir($mod_dh);
     closedir($mod_dh);
 
     for my $mod_file (@module_files) {
         my $module_name = $mod_file;
-        $module_name =~ s/\.yaml$//;
+        $module_name =~ s/\.(yaml|pup)$//;
 
         eval { $self->compile_recipe($module_name); };
         if ($@) {
